@@ -234,6 +234,8 @@ class Loader:
 
         self._indexcache = {}  # type: Dict[Any, Callable[[Loader, Any, Any], Any]]
 
+        self._objfieldscache = {}  # type: Dict[Type, Tuple[Set[str], Set[str], Dict[str, Type], Dict[str, str]]]
+
         self._unionload_discriminatorcache = {}  # type: Dict[Type, Tuple[Optional[str], Optional[Dict[Any, Type]]]]
 
     def index(self, type_: Type[T]) -> int:
@@ -364,6 +366,10 @@ def _dictload(l: Loader, value: Any, type_) -> Dict:
     """
     key_type, value_type = type_.__args__
 
+
+    key_type_basic = key_type in l.basictypes
+    value_type_basic = value_type in l.basictypes
+
     key_handler = l._indexcache.get(key_type)
     if key_handler is not None:
         key_f = key_handler
@@ -396,7 +402,9 @@ def _dictload(l: Loader, value: Any, type_) -> Dict:
     # Try fast load
     try:
         return {
-            key_f(l, k, key_type): value_f(l, v, value_type) for k, v in value.items()
+            k if key_type_basic and isinstance(k, key_type) else key_f(l, k, key_type): \
+            v if value_type_basic and isinstance(v, value_type) else value_f(l, v, value_type) \
+            for k, v in value.items()
         }
     except Exception:
         # Failed, do the slow method with exception tracking
@@ -418,14 +426,34 @@ def _tupleload(l: Loader, value: Any, type_) -> Tuple:
 
     if len(args) == 2 and args[1] == ...: # Tuple[something, ...]
         return _iterload(l, value, type_, tuple)
-    else: # Tuple[something, something, somethingelse]
-        if isinstance(value, dict):
-            raise TypedloadTypeError('Unable to load dictionary as a tuple', value=value, type_=type_)
-        if l.failonextra and len(value) > len(args):
-            raise TypedloadValueError('Value is too long for type %s' % tname(type_), value=value, type_=type_)
-        elif len(value) < len(args):
-            raise TypedloadValueError('Value is too short for type %s' % tname(type_), value=value, type_=type_)
-        return tuple(l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, (v, t) in enumerate(zip(value, args)))
+
+    # Tuple[something, something, somethingelse]
+    if isinstance(value, dict):
+        raise TypedloadTypeError('Unable to load dictionary as a tuple', value=value, type_=type_)
+    if l.failonextra and len(value) > len(args):
+        raise TypedloadValueError('Value is too long for type %s' % tname(type_), value=value, type_=type_)
+    elif len(value) < len(args):
+        raise TypedloadValueError('Value is too short for type %s' % tname(type_), value=value, type_=type_)
+
+    ctr = count(1)  # Keep track of the position in the tuple
+
+    try:
+        return tuple(v if t in l.basictypes and type(v) == t else h(l, v, t)
+            for v, h, t in zip(
+                compress(value, ctr),
+                (l._indexcache.get(t) or l.handlers[l.index(t)][1] for t in args),
+                args
+            )
+        )
+    except TypedloadException as e:
+        index = next(ctr) - 2
+        annotation = Annotation(AnnotationType.INDEX, index)
+        e.trace.insert(0, TraceItem(value, type_, annotation))
+        raise e
+    except TypeError as e:
+        raise TypedloadTypeError(str(e), value=value, type_=type_)
+    except Exception as e:
+        raise TypedloadTypeError('Exception is not a subclass of TypedloadException. Make sure all handlers only raise TypedloadException')
 
 
 def _mangle_names(namesmap: Dict[str, str], value: Dict[str, Any], failonextra: bool) -> Dict[str, Any]:
@@ -459,26 +487,30 @@ def _dataclassload(l: Loader, value: Dict[str, Any], type_) -> Any:
     """
     This loads a Dict[str, Any] into a NamedTuple.
     """
-    from dataclasses import _MISSING_TYPE as DT_MISSING_TYPE
-    fields = set(type_.__dataclass_fields__.keys())
-    necessary_fields = {k for k,v in type_.__dataclass_fields__.items() if
-                        v.init == True and
-                        isinstance(v.default, DT_MISSING_TYPE) and
-                        isinstance(v.default_factory, DT_MISSING_TYPE)}
-    if l.pep563:
-        type_hints = get_type_hints(type_)
+    cached = l._objfieldscache.get(type_)
+    if cached:
+        fields, necessary_fields, type_hints, transforms = cached
     else:
-        type_hints = {k: v.type for k,v in type_.__dataclass_fields__.items()}
+        fields = set(type_.__dataclass_fields__.keys())
+        necessary_fields = {k for k,v in type_.__dataclass_fields__.items() if
+                            v.init == True and # Is a field for the constructor
+                            v.default == v.default_factory # Has no default or factory
+                            }
+        if l.pep563:
+            type_hints = get_type_hints(type_)
+        else:
+            type_hints = {k: v.type for k,v in type_.__dataclass_fields__.items()}
 
-    #Name mangling
+        #Name mangling
 
-    # Prepare the list of the needed name changes
-    transforms = {}  # type: Dict[str, str]
-    for pyname in fields:
-        if type_.__dataclass_fields__[pyname].metadata:
-            name = type_.__dataclass_fields__[pyname].metadata.get(l.mangle_key)
-            if name:
-                transforms[name] = pyname
+        # Prepare the list of the needed name changes
+        transforms = {}
+        for pyname in fields:
+            if type_.__dataclass_fields__[pyname].metadata:
+                name = type_.__dataclass_fields__[pyname].metadata.get(l.mangle_key)
+                if name:
+                    transforms[name] = pyname
+        l._objfieldscache[type_] = (fields, necessary_fields, type_hints, transforms)
 
     try:
         value = _mangle_names(transforms, value, l.failonextra)
@@ -531,24 +563,40 @@ def _objloader(l: Loader, fields: Set[str], necessary_fields: Set[str], type_hin
             type_=type_,
         )
 
+    if l.failonextra and len(extra_fields := vfields.difference(fields)):
+        extra = ', '.join(extra_fields)
+        raise TypedloadValueError(
+            'Dictionary has unrecognized fields: %s and cannot be loaded into %s' % (extra, tname(type_)),
+            value=value,
+            type_=type_,
+        )
+
     params = {}
     for k, v in value.items():
         if k not in fields:
             # Field in value is not in the type
-            if l.failonextra:
-                extra = ', '.join(vfields.difference(fields))
-                raise TypedloadValueError(
-                    'Dictionary has unrecognized fields: %s and cannot be loaded into %s' % (extra, tname(type_)),
+            continue
+
+        # loading field directly, skipping load()
+        field_type = type_hints[k]
+        cached_loader = l._indexcache.get(field_type)
+        if cached_loader:
+            loader_f = cached_loader
+        else:
+            try:
+                loader_f = l._indexcache[field_type] = l.handlers[l.index(field_type)][1]
+            except ValueError:
+                raise TypedloadTypeError(
+                    'Cannot deal with value of type %s' % tname(field_type),
                     value=value,
-                    type_=type_,
+                    type_=field_type
                 )
-            else:
-                continue
-        params[k] = l.load(
-            v,
-            type_hints[k],
-            annotation=Annotation(AnnotationType.FIELD, k),
-        )
+        try:
+            params[k] = loader_f(l, v, field_type)
+        except TypedloadException as e:
+            annotation=Annotation(AnnotationType.FIELD, k)
+            e.trace.insert(0, TraceItem(value, type_, annotation))
+            raise e
     try:
         return type_(**params)
     except TypeError as e:
@@ -661,12 +709,28 @@ def _unionload(l: Loader, value: Any, type_) -> Any:
     r = None
     for t in sorted_args:
         try:
-            r = l.load(value, t, annotation=Annotation(AnnotationType.UNION, t))
+            # Skip calling load()
+            handler = l._indexcache.get(t)
+            if handler is not None:
+                f = handler
+            else:
+                try:
+                    f = l._indexcache[t] = l.handlers[l.index(t)][1]
+                except ValueError:
+                    raise TypedloadValueError(
+                        'Cannot deal with value of type %s (key of %s)' % (tname(t), tname(type_)),
+                        value=value,
+                        type_=t
+                    )
+
+            r = f(l, value, t)
             loaded_count += 1
             if not l.uniondebugconflict:
                 # Do not try more if we are not debugging
                 break
         except TypedloadException as e:
+            annotation = Annotation(AnnotationType.UNION, t)
+            e.trace.insert(0, TraceItem(value, type_, annotation))
             exceptions.append(e)
 
     if loaded_count == 1:
@@ -838,17 +902,20 @@ def _iterload(l: Loader, value: Any, type_, function) -> Any:
             f = l._indexcache[t] = l.handlers[l.index(t)][1]
         except ValueError:
             raise TypedloadTypeError(
-                'Cannot deal with value of type %s' % tname(type_),
+                'Cannot deal with value of type %s' % tname(t),
                 value=value,
-                type_=type_
+                type_=t,
             )
 
-
-
     # load calling the handler directly, skipping load()
     try:
         ctr = count(1)
-        return function(map(f, repeat(l), compress(value, ctr), repeat(t)))
+        if t in l.basictypes:
+            return function((i if isinstance(i, t) else f(l, i, t) for i in compress(value, ctr)))
+        elif is_union(t) and (types := set(uniontypes(t))).issubset(l.basictypes):
+            return function((i if type(i) in types else f(l, i, t) for i in compress(value, ctr)))
+        else:
+            return function(map(f, repeat(l), compress(value, ctr), repeat(t)))
     except TypedloadException as e:
         index = next(ctr) - 2
         annotation = Annotation(AnnotationType.INDEX, index)
